\name{rgl.bbox}
\alias{rgl.bbox}
\title{setup Bounding Box decoration}
\description{
  Setup the bounding box decoration.
}
\usage{
rgl.bbox( 
  xat=NULL, xlab=NULL, xunit=0, xlen=5, 
  yat=NULL, ylab=NULL, yunit=0, ylen=5,
  zat=NULL, zlab=NULL, zunit=0, zlen=5,
  marklen=15.0, marklen.rel=TRUE, ...)
}
\arguments{
  \item{xat,yat,zat}{vector specifying the tickmark positions}
  \item{xlab,ylab,zlab}{character vector specifying the tickmark labeling}
  \item{xunit,yunit,zunit}{value specifying the tick mark base for uniform tick mark layout}
  \item{xlen,ylen,zlen}{value specifying the number of tickmarks}
  \item{marklen}{value specifying the length of the tickmarks}
  \item{marklen.rel}{logical, if TRUE tick mark length is calculated using 1/\code{marklen} * axis length, otherwise tick mark length is \code{marklen} in coordinate space}
  \item{ ... }{Material properties. See \code{\link{rgl.material}} for details.}
}
\details{
  Three different types of tick mark layouts are possible.
  If at is not NULL, the ticks are setup at custom positions.
  If unit is not zero, it defines the tick mark base.
  If length is not zero, it specifies the number of ticks that are automatically specified.
  The first colour specifies the bounding box, while the second one specifies the tick mark and font colour.
}
\examples{
  rgl.bbox(color="#333377", emission="#333377", specular="#3333FF", shininess=5, alpha=0.8 )
}
\seealso{
  \code{\link{rgl.material}}
}
\keyword{dynamic}

\eof
\name{rgl.bg}
\alias{rgl.bg}
\title{setup Background}
\description{
  Setup the background environment of the scene.
}
\usage{
rgl.bg( sphere = FALSE, fogtype = "none", color=c("black","white"), back="lines", ...)
}
\arguments{
  \item{fogtype}{fog type:
    \itemize{
      \item \code{"none"}   no fog
      \item \code{"linear"} linear fog function
      \item \code{"exp"}    exponential fog function
      \item \code{"exp2"}   squared exponential fog function
    }
  }
  \item{sphere}{
  logical, if true, an environmental sphere geometry is used for the background decoration.
  }
  \item{color}{
  Primary color is used for background clearing.
  Secondary color is used for sphere geometry. See \code{\line{rgl.material}} for details.
  }
  \item{back}{
  Specifies the fill style of the sphere geometry. See \code{\line{rgl.material}} for details.
  }
  \item{ ... }{Material properties. See \code{\link{rgl.material}} for details.}  
}
\details{
  If sphere is set to TRUE, an environmental sphere enclosing the whole scene is drawn.
}
\examples{

  # the holo-globe (inspired by star trek):

  rgl.bg(sphere=TRUE, color=c("black","green"), lit=FALSE, back="lines" )

  # an environmental sphere with a nice texture.

  rgl.bg(sphere=TRUE, texture=system.file("textures/sunsleep.png", package="rgl"), back="filled" )
}
\seealso{
  \code{\link{rgl.material}}
}
\keyword{dynamic}

\eof
\name{rgl.light}
\alias{rgl.light}
\title{add light source}
\description{
  rgl.light adds a light source to the scene.
}
\usage{
rgl.light( theta = 0, phi = 0, viewpoint.rel = TRUE, ambient = "#FFFFFF", diffuse = "#FFFFFF", specular = "#FFFFFF")
}
\arguments{
  \item{theta, phi}{polar coordinates}
  \item{viewpoint.rel}{logical, if TRUE light is a viewpoint light that is positioned relative to the current viewpoint}
  \item{ambient, diffuse, specular } { light color values used for lighting calculation }
}
\details{
  Up to 8 light sources are supported. They are positioned either in world space
  or relative to the camera using polar coordinates. Light sources are directional.
}
\seealso{
  \code{\link{rgl.clear}}
  \code{\link{rgl.pop}}
}
\keyword{dynamic}

\eof
\name{rgl.material}
\alias{rgl.material}
\title{generic Appearance setup}
\description{
  Set material properties for geometry appearance.
}
\usage{
rgl.material(
  color        = c("white"),
  alpha        = c(1.0),
  lit          = TRUE, 
  ambient      = "black",
  specular     = "white", 
  emission     = "black", 
  shininess    = 50.0, 
  smooth       = TRUE,
  texture      = NULL, 
  textype      = "rgb", 
  texmipmap    = FALSE, 
  texminfilter = "linear", 
  texmagfilter = "linear",  
  front        = "fill", 
  back         = "fill",
  size         = 1.0, 
  fog          = TRUE
)
}
\arguments{
  \item{color}{
    vector of R color characters. Represents the diffuse component in case of lighting calculation (lit = TRUE),
    otherwise it describes the solid color characteristics.
  }
  \item{lit}{
    logical, specifying if lighting calculation should take place on geometry
  }
  \item{ambient, specular, emission, shininess}{
    properties for lighting calculation. ambient, specular, emission are R color character string values; shininess represents a
    numerical.
  }
  \item{alpha}{ 
    vector of alpha values between 0.0 (fully transparent) .. 1.0 (opaque).
  }
  \item{smooth}{
    logical, specifying whether gourad shading (smooth) or flat shading should be used.
  }
  \item{texture}{ 
    path to a texture image file. Supported formats: png.
  }
  \item{textype}{
    specifies what is defined with the pixmap
    \describe{
      \item{"alpha"}{alpha values}
      \item{"luminance"}{luminance}
      \item{"luminance.alpha"}{luminance and alpha}
      \item{"rgb"}{color}
      \item{"rgba"}{color and alpha texture}
    }
  }
  \item{texmipmap}{
    Logical, specifies if the texture should be mipmapped.
  }
  \item{texmagfilter}{
    specifies the magnification filtering type (sorted by ascending quality):
    \describe{
      \item{"nearest"}{texel nearest to the center of the pixel}
      \item{"linear"}{weighted linear average of a 2x2 array of texels}
    }
  }
  \item{texminfilter}{
    specifies the minification filtering type (sorted by ascending quality):
    \describe{
      \item{"nearest"}{texel nearest to the center of the pixel}
      \item{"linear"}{weighted linear average of a 2x2 array of texels}
      \item{"nearest.mipmap.nearest"}{low quality mipmapping}
      \item{"nearest.mipmap.linear"}{medium quality mipmapping}
      \item{"linear.mipmap.nearest"}{medium quality mipmapping}
      \item{"linear.mipmap.linear"}{high quality mipmapping}
    }
  }
  \item{front, back}{
    Determines the polygon mode for the specified side:
    \describe{
      \item{"fill"}{filled polygon}
      \item{"line"}{wireframed polygon}
      \item{"points"}{point polygon}
      \item{"cull"}{culled (hidden) polygon}
    }
  }
  \item{size}{ 
    numeric, specifying the line and point size.
  }
  \item{fog}{logical, specifying if fog effect should be applied on the corresponding shape}
}
\details{
  Only one side at a time can be culled.
}
\seealso{
\code{\link{rgl.primitive}},
\code{\link{rgl.bbox}},
\code{\link{rgl.bg}},
\code{\link{rgl.light}}
}
\keyword{dynamic}


\eof
\name{rgl.primitive}
\alias{rgl.primitive}
\alias{rgl.points}
\alias{rgl.lines}
\alias{rgl.triangles}
\alias{rgl.quads}
\title{add primitive set shape}
\description{
  Adds a shape node to the current scene
}
\usage{
rgl.points(x, y, z, ... )
rgl.lines(x, y, z, ... )
rgl.triangles(x, y, z, ... )
rgl.quads(x, y, z, ... )
}
\arguments{
  \item{x, y, z}{coordinates}
  \item{ ... }{Material properties. See \code{\link{rgl.material}} for details.}  
}
\details{
  Adds a shape node to the scene. The appearance is defined by the material properties.
  See \code{\link{rgl.material}} for details.
}
\examples{
rgl.clear()
rgl.points(rnorm(1000), rnorm(1000), rnorm(1000), color=heat.colors(1000), size=2)
}
\seealso{
\code{\link{rgl.material}},
\code{\link{rgl.spheres}},
\code{\link{rgl.texts}},
\code{\link{rgl.surface}},
\code{\link{rgl.sprites}}
}
\keyword{dynamic}

\eof
\name{rgl-internal}
\title{Internal rgl functions}
\alias{rgl.bool}
\alias{rgl.numeric}
\alias{rgl.range}
\alias{rgl.vertex}
\alias{rgl.nvertex}
\alias{rgl.color}
\alias{rgl.mcolor}
\alias{rgl.clamp}
\alias{rgl.attr}
\alias{rgl.enum}
\alias{rgl.enum.nodetype}
\alias{rgl.enum.pixfmt}
\alias{rgl.enum.polymode}
\alias{rgl.enum.textype}
\alias{rgl.enum.fogtype}
\alias{rgl.enum.primtype}
\alias{rgl.enum.halign}
\alias{rgl.enum.texmagfilter}
\alias{rgl.enum.texminfilter}
\description{
  internal rgl functions
}
\usage{
rgl.bool(x)
rgl.numeric(x)
rgl.range(x, low, high)
rgl.vertex(x,y,z)
rgl.nvertex(vertex)
rgl.color(color)
rgl.mcolor(colors)
rgl.clamp(value, low, high)
rgl.attr(vattr, nvertex) 
rgl.enum(name, ...)
rgl.enum.nodetype(type)
rgl.enum.pixfmt(fmt)
rgl.enum.polymode(mode)
rgl.enum.textype(textype)
rgl.enum.fogtype(fogtype)
rgl.enum.primtype(primtype)
rgl.enum.halign(halign)
rgl.enum.texmagfilter(magfiltertype)
rgl.enum.texminfilter(minfiltertype)
}
\details{
  These are not to be called by the user.
}
\keyword{internal}

\eof
\name{rgl}
\title{3D visualization device system}
\alias{rgl}
\alias{rgl.open}
\alias{rgl.close}
\alias{rgl.cur}
\alias{rgl.set}
\alias{rgl.quit}
\description{
  3D real-time rendering device-driver system
}
\usage{
rgl.open()     # open new device
rgl.close()    # close current device
rgl.cur()      # returns active device id
rgl.set(which) # set device as active
rgl.quit()     # shutdown rgl device system
}
\arguments{
  \item{which}{device id}
}
\details{
  RGL is a 3D real-time rendering device driver system for R.
  Multiple devices are managed at a time, where one has the current focus
  that receives instructions from the R command-line.
  The device design is oriented towards the R device metaphor. If you send
  scene management instructions, and there's no device open, it will be opened
  automatically.
  Opened devices automatically get the current device focus. The focus may be
  changed by using rgl.set().
  rgl.quit() shuts down the rgl subsystem and all open devices, 
  detaches the package including the shared library and additional system libraries.
}
\examples{
example(rgl.surface)
}
\seealso{
\code{\link{rgl.clear}}, 
\code{\link{rgl.pop}},
\code{\link{rgl.viewpoint}},
\code{\link{rgl.light}},
\code{\link{rgl.bg}},
\code{\link{rgl.bbox}},
\code{\link{rgl.points}},
\code{\link{rgl.lines}},
\code{\link{rgl.triangles}},
\code{\link{rgl.quads}},
\code{\link{rgl.texts}},
\code{\link{rgl.surface}},
\code{\link{rgl.spheres}},
\code{\link{rgl.sprites}},
\code{\link{rgl.snapshot}}
}
\keyword{dynamic}

\eof
\name{rgl.clear}
\alias{rgl.clear}
\alias{rgl.pop}
\title{scene management}
\description{
Clear shapes, lights, bbox
}
\usage{
rgl.clear( type = "shapes" )
rgl.pop( type = "shapes" )  
}
\arguments{
  \item{type}{Select subtype:
    \describe{
      \item{"shapes"}{shape stack}
      \item{"lights"}{light stack}
      \item{"bbox"}{bounding box}
    }
  }
}
\details{
RGL holds two stacks. One is for shapes and the other is for lights. 
\code{rgl.clear} clears the specified stack, while \code{rgl.pop} removes 
the top-most (last added) node on the stack.
Since the bounding box is an optional element, one can disable it 
with \code{rgl.clear}.
}
\seealso{
\code{\link{rgl}}
\code{\link{rgl.bbox}}
\code{\link{rgl.light}}
}
\keyword{dynamic}

\eof
\name{rgl.snapshot}
\alias{rgl.snapshot}
\title{export screenshot}
\description{
  Saves the screenshot as png file.
}
\usage{
rgl.snapshot( filename, fmt="png" )
}
\arguments{
  \item{filename}{full path to filename.}
  \item{fmt}{image export format, currently supported: png }
}
\details{
Animations can be created in a loop modifying the scene and saving 
a screenshot to a file. (See example below)
}
\examples{

\dontrun{

#
# create animation
#

for (i in 1:45) {
  rgl.viewpoint(i,20)
  print( paste("pic",formatC(i,digits=1,flag="0"),".png",sep="") )
  rgl.snapshot(filename)
}

}

}
\seealso{
  \code{\link{rgl.viewpoint}}
}
\keyword{dynamic}

\eof
\name{rgl.spheres}
\alias{rgl.spheres}
\title{add sphere set shape}
\description{
  Adds a sphere set shape node to the scene
}
\usage{
rgl.spheres(x, y, z, radius, ...)
}
\arguments{
  \item{x, y, z}{Point coordinates}
  \item{radius}{Vector or singe value defining the sphere radius}
  \item{ ... }{Material properties. See \code{\link{rgl.material}} for details.}
}
\details{
}
\examples{
rgl.spheres(rnorm(10), rnorm(10), rnorm(10), radius=.2, color=rainbow(10))
}
\seealso{
  \code{\link{rgl.material}}
}
\keyword{dynamic}

\eof
\name{rgl.sprites}
\alias{rgl.sprites}
\title{add sprite set shape}
\description{
  Adds a sprite set shape node to the scene.
}
\usage{
rgl.sprites(x, y, z, radius, ...)
}
\arguments{
  \item{ x, y, z }{point coordinates}
  \item{ radius } {vector or singe value defining the sphere radius}
  \item{ ... }{material properties, texture mapping is supported}
}
\details{
  Sprites are rectangle planes that are directed towards the viewpoint.
  Its primary used for fast (and faked) atmospherical effects, e.g. particles and clouds
  using alpha blended textures.
}
\examples{
rgl.sprites(rnorm(500), rnorm(500), rnorm(500), color=rainbow(100), lit=FALSE,
            textype="alpha", texture=system.file("textures/particle.png", package="rgl"), 
            alpha=.2)
}
\seealso{
  \code{\link{rgl.material}}
}
\keyword{dynamic}

\eof
\name{rgl.surface}
\title{add height-field surface shape}
\alias{rgl.surface}
\description{
  Adds a surface to the current scene. The surface is defined by two
  grid mark vectors and a matrix defining the height of each grid point.
}
\usage{
rgl.surface(x, z, y, ...)
}
\arguments{
  \item{ x, z }{
    x,z axis tick marks
  }
  \item{ y }{
    y height values
  }
  \item{ ... }{Material and texture properties. See \code{\link{rgl.material}} for details.}    
}
\details{
  Adds a surface mesh to the current scene. The surface is evaluated by 
  defining the x- and z-axis tick marks for the control grid. The
  matrix y defines the height values for each point.
}
\examples{

#
# volcano example taken from "persp"
#

data(volcano)

y <- 2 * volcano        # Exaggerate the relief

x <- 10 * (1:nrow(y))   # 10 meter spacing (S to N)
z <- 10 * (1:ncol(y))   # 10 meter spacing (E to W)

ylim <- range(y)
ylen <- ylim[2] - ylim[1] + 1

colorlut <- terrain.colors(ylen) # height color lookup table

col <- colorlut[ y-ylim[1]+1 ] # assign colors to heights for each point

rgl.clear()
rgl.surface(x, z, y, color=col)

}
\seealso{
  \code{\link{rgl.material}}
}
\keyword{dynamic}

\eof
\name{rgl.texts}
\alias{rgl.texts}
\title{add text set shape}
\description{
  Adds text to the scene. The text is positioned in 3D space. 
  A bitmap font which is always oriented towards the camera is used.
}
\usage{
rgl.texts(x, y, z, text, justify, ...)
}
\arguments{
  \item{x, y, z}{point coordinates}
  \item{text}{text character vector to draw}
  \item{justify}{ character string specifying the horizontal adjustment,
    can be one of \code{"left"}, \code{"right"}, \code{"center"}.
  }
  \item{ ... }{Material properties. See \code{\link{rgl.material}} for details.}  
}
\details{
}
\examples{
rgl.texts(rnorm(10)*100,rnorm(10)*100,rnorm(10)*100,text=1:10,justify="center", color=heat.colors(10))
}
\seealso{
  \code{\link{rgl.material}}
}
\keyword{dynamic}

\eof
\name{rgl.viewpoint}
\alias{rgl.viewpoint}
\title{setup viewpoint}
\description{
  Set the viewpoint orientation.
}
\usage{
rgl.viewpoint( theta = 0, phi = 15, fov = 60, zoom = 0, interactive = TRUE )
}
\arguments{
  \item{theta,phi}{polar coordinates}
  \item{fov}{field-of-view angle}
  \item{zoom}{zoom factor}
  \item{interactive}{logical, specifying if interactive navigation is allowed}
}
\details{
The viewpoint is always set in an orbit around the data model. 
It can be either specified with \code{rgl.viewpoint} or interactively using the pointing device of
your graphics user-interface.
With the pointing device you can use the buttons as follows:

\itemize{
\item{left}{adjust viewpoint position}
\item{middle}{adjust field of view angle}
\item{right or wheel}{adjust zoom factor}
}

}
\examples{

# animated round trip tour

for(i in 1:360) {
  rgl.viewpoint(i,i/4); 
}

}
\keyword{dynamic}

\eof
